import React, { useMemo, useState, forwardRef, useImperativeHandle, useRef } from 'react'
import { Table, Button, Popconfirm, Input, Space, Tag, Popover, Checkbox, Form } from 'antd'
import { TweenOneGroup } from 'rc-tween-one'
import styles from './index.less'
import { SearchOutlined, FormOutlined } from '@ant-design/icons'
import * as icons from '@ant-design/icons'
import Text from './components/Text'
import ResizeableTitle from './components/ResizeableTitle'
import FileView from './components/FileView'
import CoordView from './components/CoordView'
import SelectView from './components/SelectView'
import ValueEdit from './components/ValueEdit'
import IconPack from '../../../components/IconPack'
import { isObject, isArray, getFieldInfo } from '../../../../utils'

const iconList = Object.keys(icons).filter((item) => typeof icons[item] === 'object')

const TablePack = (props, ref) => {

  useImperativeHandle(ref, () => ({
    setFilteredInfo,
    setSortedInfo,
    fileColumns,
  }))

  const { notUse, config, fieldList, loading, dataSource, sourceChange, rowSelection, onRow, tableChange, btnsClick, isEdit } = props
  const { accountFieids, webShowFieldGroup, formJson, parent } = config
  const [fieldResize, setFieldResize] = useState({})
  const [filteredInfo, setFilteredInfo] = useState({})
  const [sortedInfo, setSortedInfo] = useState({})
  const [open, setOpen] = useState(false)
  const [checks, setChecks] = useState([])
  const searchInput = useRef(null)
  const [form] = Form.useForm()

  const filtered = useMemo(() => {
    let array = []
    if (isObject(filteredInfo)) {
      Object.keys(filteredInfo).forEach(v => {
        if (Array.isArray(filteredInfo[v])) {
          array.push({ filterName: v, filterValue: filteredInfo[v].join(',') })
        }
      })
    }
    return array
  }, [filteredInfo])

  //台账列表、台账选择器表头
  const getColumnProps = (json, v) => {
    const { fieldName, columnWidth, isSort, accurateSearch, alignment } = v
    const { widget, sourceType, options, addonAfter } = json?.[fieldName] || {}
    let searchProps = {}
    // console.log('json?.[fieldName]', json?.[fieldName])
    if (accurateSearch) {
      if (['CheckBox', 'ComboBox', 'RadioButton'].includes(widget)) {
        if (sourceType === '手动输入') {
          searchProps = {
            filters: options.map(v => { return { text: v.value, value: v.value, } }),
          }
        }
      } else {
        searchProps = {
          filterDropdown: ({ setSelectedKeys, selectedKeys, confirm, clearFilters, close }) => {
            return (
              <div
                style={{ padding: 8 }}
                onKeyDown={(e) => e.stopPropagation()}
              >
                <Input
                  placeholder={`搜索 ${fieldName}`}
                  value={selectedKeys[0]}
                  onChange={e => {
                    setSelectedKeys(e.target.value ? [e.target.value] : [])
                  }}
                  onPressEnter={() => {
                    confirm({ closeDropdown: true })
                  }}
                  style={{
                    marginBottom: 8,
                    display: 'block',
                    width: '280px',
                  }}
                />
                <Space>
                  <Button
                    type="primary"
                    onClick={() => {
                      confirm({ closeDropdown: true })
                    }}
                    icon={<SearchOutlined />}
                    size="small"
                    style={{ width: 90 }}
                  >
                    搜索
                  </Button>
                  <Button
                    onClick={() => {
                      clearFilters()
                      confirm({ closeDropdown: true })
                    }}
                    size="small"
                    style={{ width: 90 }}
                  >
                    重置
                  </Button>
                </Space>
              </div>
            )
          },
          filterIcon: (filtered) => (
            <SearchOutlined
              style={{
                color: filtered ? '#1890ff' : undefined,
              }}
            />
          ),
        }
      }
    }
    return {
      title: (
        <div className={styles.headerTitle}>
          {json[fieldName]?.title || fieldName}
          {addonAfter && !iconList.includes(addonAfter) ? `(${addonAfter})` : null}
        </div>
      ),
      dataIndex: fieldName,
      width: fieldResize[fieldName] || columnWidth || 120,
      ellipsis: true,
      sorter: Boolean(isSort),
      sortOrder: sortedInfo.field === fieldName ? sortedInfo.order : null,
      filteredValue: filteredInfo[fieldName] || null,
      ...searchProps,
      align: alignment === '中' ? 'center' : (alignment === '右' ? 'right' : 'left'),
      onHeaderCell: (column) => ({
        width: column.width,
        onResize: (e, props) => {
          setFieldResize({ ...fieldResize, [fieldName]: props?.size?.width })
        },
      }),
      render: (value, record) => {
        let props = { ...json[fieldName], value }
        if (widget === 'FileUpload') {
          return <FileView {...props} />
        }
        if (['Coordinate', 'Device', 'DrawPath', 'DrawArea'].includes(widget)) {
          return <CoordView {...props} />
        }
        if (['RadioButton', 'CheckBox'].includes(widget)) {
          return <SelectView {...props} />
        }
        return <Text align={alignment} text={value} />
      },
    }
  }

  //关联表单表头
  const getRelevanceColumnProps = ({ json, field, isEdit, autoArray }) => {
    const { fieldName, columnWidth, isSort, accurateSearch, alignment } = field
    const { widget } = json?.[fieldName] || {}
    let searchProps = {}
    if (accurateSearch) {
      searchProps = {
        filterDropdown: ({ setSelectedKeys, selectedKeys, confirm, clearFilters, close }) => (
          <div
            style={{ padding: 8 }}
            onKeyDown={(e) => e.stopPropagation()}
          >
            <Input
              ref={searchInput}
              placeholder={`Search ${fieldName}`}
              value={selectedKeys[0]}
              onChange={(e) => setSelectedKeys(e.target.value ? [e.target.value] : [])}
              onPressEnter={() => {
                confirm();
              }}
              style={{
                marginBottom: 8,
                display: 'block',
              }}
            />
            <Space>
              <Button
                type="primary"
                onClick={() => {
                  confirm();
                }}
                icon={<SearchOutlined />}
                size="small"
                style={{
                  width: 90,
                }}
              >
                搜索
              </Button>
              <Button
                onClick={() => {
                  clearFilters();
                  confirm();
                }}
                size="small"
                style={{
                  width: 90,
                }}
              >
                重置
              </Button>
            </Space>
          </div>
        ),
        filterIcon: (filtered) => (
          <SearchOutlined
            style={{
              color: filtered ? '#1890ff' : undefined,
            }}
          />
        ),
        onFilter: (value, record) =>
          record[fieldName].toString().toLowerCase().includes(value.toLowerCase()),
        onFilterDropdownOpenChange: (visible) => {
          if (visible) {
            setTimeout(() => searchInput.current?.select(), 100);
          }
        },
      }
    }
    return {
      title: json[fieldName]?.title || fieldName,
      dataIndex: fieldName,
      width: fieldResize[fieldName] || columnWidth || 120,
      ellipsis: true,
      sorter: Boolean(isSort) ? (a, b) => a[fieldName] - b[fieldName] : false,
      filteredValue: filteredInfo[fieldName] || null,
      ...searchProps,
      align: alignment === '中' ? 'center' : (alignment === '右' ? 'right' : 'left'),
      onHeaderCell: (column) => ({
        width: column.width,
        onResize: (e, props) => {
          setFieldResize({ ...fieldResize, [fieldName]: props?.size?.width })
        },
      }),
      render: (value, record) => {
        let props = { ...json[fieldName], fieldName, value, autoArray, record: { ...record } }
        if (isEdit && !notUse?.includes('edit')) {
          return (
            <ValueEdit
              {...props}
              onChange={({ fieldName, fieldValue }) => {
                btnsClick({ type: '变化', row: { ...record, [fieldName]: fieldValue } })
              }}
              otherChange={(otherRow) => {
                btnsClick({ type: '变化', row: { ...record, ...otherRow } })
              }}
              value={value}
            />
          )
        } else {
          if (['FileUpload'].includes(widget)) {
            return <FileView {...props} />
          }
          if (['Coordinate', 'Device', 'DrawPath', 'DrawArea'].includes(widget)) {
            return <CoordView {...props} />
          }
          if (['RadioButton', 'CheckBox'].includes(widget)) {
            return <SelectView {...props} />
          }
          return <Text text={value} />
        }
      },
    }
  }

  const btns = useMemo(() => {
    let array = []
    //表格可编辑
    if (isEdit) {
      if (!notUse?.includes('del')) {
        array.push({ title: '删除', icon: <IconPack.ShanChu /> })
      }
    } else {
      if (!notUse?.includes('read')) {
        array.push({ title: '详情', icon: <IconPack.XiangQing />, })
      }
      if (!notUse?.includes('edit')) {
        array.push({ title: '编辑', icon: <IconPack.BianJi /> })
      }
      if (!notUse?.includes('del')) {
        array.push({ title: '删除', icon: <IconPack.ShanChu /> })
      }
    }

    return array
  }, [notUse, isEdit])

  const checkChange = (values) => {
    setChecks(values)
  }

  const showField = useMemo(() => {
    let showField = []
    accountFieids.forEach(v => {
      if (isArray(fieldList)) {
        if (fieldList.includes(v.fieldName)) {
          showField.push(v)
        }
      } else {
        if (v.webDisplay) {
          showField.push(v)
        }
      }
    })
    setChecks(showField.map(v => v.fieldName))
    return showField
  }, [accountFieids, fieldList])

  const fileColumns = useMemo(() => {
    let json = getFieldInfo(formJson)
    let autoArray = []
    Object.keys(json).forEach(key => {
      let item = json[key]
      if (item.widget === 'NumberInput') {
        if (item.calculateRule) {
          let parmas = {
            fields: item?.rules[0]?.fields,
            tableName: item.tableNameParent,
            filedFormulas: [
              {
                fieldName: key,
                formula: item.calculateRule
              }
            ]
          }
          autoArray.push(parmas)
        }
      }
    })
    let array = []
    showField.forEach((v, i) => {
      if (parent === '关联表单') {
        // array.push(getRelevanceColumnProps(json, v, isEdit))
        array.push(getRelevanceColumnProps({ json, field: v, isEdit, autoArray }))
      } else {
        array.push(getColumnProps(json, v))
      }
    })
    return array
  }, [parent, formJson, showField, fieldResize, filteredInfo, sortedInfo, isEdit, dataSource])

  const showColumns = useMemo(() => {
    return fileColumns.filter(v => checks.includes(v.dataIndex))
  }, [checks, fileColumns])

  const content = (
    <div>
      <Checkbox.Group
        value={checks}
        style={{ width: '100%' }}
        onChange={checkChange}
      >
        {
          fileColumns.map(v => {
            return (
              <div key={v.dataIndex}>
                <Checkbox value={v.dataIndex}>{v.title}</Checkbox>
              </div>
            )
          })
        }
      </Checkbox.Group>
    </div>
  )

  const columns = useMemo(() => {
    let columns = [
      {
        title: '序号',
        width: 50,
        align: 'center',
        filteredValue: null,
        render: (_, r, i) => i + 1
      },
      ...showColumns,
    ]
    if (btns.length) {
      columns.push(
        {
          title: (
            <div className={styles.operation}>
              <span>操作</span>
              <span className={styles.operationSpan}>
                <Popover
                  placement='bottomRight'
                  title={
                    <div style={{ display: 'flex', justifyContent: 'space-between', alignItems: 'center' }}>
                      <div>列配置</div>
                      <Button size='small' type="link" onClick={() => setOpen(false)}>确定</Button>
                    </div>
                  }
                  trigger='click'
                  open={open}
                  content={content}
                  onOpenChange={(value) => {
                    setOpen(value)
                  }}
                  arrowPointAtCenter
                >
                  {
                    fileColumns.length ? <FormOutlined /> : null
                  }
                </Popover>
              </span>
            </div>
          ),
          width: btns.length * 35 + 30,
          align: 'center',
          filteredValue: null,
          fixed: 'right',
          render: (r) => {
            return (
              <div style={{ display: 'flex', justifyContent: 'center' }}>
                {
                  btns.map(v => {
                    if (v.title === '删除') {
                      return (
                        <Popconfirm
                          key={v.title}
                          title='确定要删除数据吗？'
                          onConfirm={() => btnsClick({ type: v.title, row: r })}
                          okText='确定'
                          cancelText='取消'
                        >
                          <Button
                            type='link'
                            title='删除'
                            style={{ padding: '0 7px' }}
                            icon={v.icon}
                          />
                        </Popconfirm>
                      )
                    }
                    return (
                      <Button
                        key={v.title}
                        type='link'
                        title={v.title}
                        icon={v.icon}
                        style={{ padding: '0 7px' }}
                        onClick={() => btnsClick({ type: v.title, row: r })}
                      >
                      </Button>
                    )
                  })
                }
              </div>
            )
          }
        }
      )
    }
    if (parent === '台账选择器') {
      return fileColumns
    }
    return columns
  }, [parent, btns, dataSource, fileColumns, checks, open])

  const onChange = (page, filters, sorter) => {
    setFilteredInfo(filters)
    setSortedInfo(sorter)
    tableChange?.(page, filters, sorter)
  }

  const closeTag = (e, { filterName, filterValue }) => {
    e.preventDefault()
    let filtered = {}
    for (let key in filteredInfo) {
      if (key === filterName) {
        filtered[key] = null
      } else {
        filtered[key] = filteredInfo[key]
      }
    }
    setFilteredInfo(filtered)
    tableChange?.({}, filtered, sortedInfo)
  }

  return (
    <div className={styles.tablePack} style={{ paddingTop: filtered.length ? '30px' : '10px' }}>
      {
        filtered.length ? (
          <div className={styles.condition}>
            <TweenOneGroup
              enter={{
                scale: 0.8,
                opacity: 0,
                type: 'from',
                duration: 200,
              }}
              onEnd={(e) => {
                if (e.type === 'appear' || e.type === 'enter') {
                  e.target.style = 'display: inline-block';
                }
              }}
              leave={{
                opacity: 0,
                width: 0,
                scale: 0,
                duration: 200,
              }}
              appear={false}
            >
              {
                filtered.map(v => {
                  return (
                    <span
                      key={v.filterName}
                      style={{ display: 'inline-block' }}
                    >
                      <Tag
                        color='blue'
                        closable
                        onClose={(e) => closeTag(e, v)}
                      >
                        <span>{v.filterName}</span>
                        <span style={{ padding: '0 3px' }}>:</span>
                        <span>{v.filterValue}</span>
                      </Tag>
                    </span>
                  )
                })
              }
            </TweenOneGroup>
          </div>
        ) : null
      }
      <Form form={form} component={false}>
        <Table
          size='small'
          rowKey='ID'
          bordered
          loading={loading}
          columns={columns}
          dataSource={dataSource}
          rowSelection={rowSelection}
          onRow={onRow}
          pagination={parent === '关联表单' ? { total: dataSource.length, showTotal: (value) => `总计 ${value} 条` } : false}
          scroll={{ x: '100%', y: 'calc(100% - 40px)' }}
          onChange={onChange}
          components={{
            header: {
              cell: ResizeableTitle
            },
          }}
        />
      </Form>
    </div>
  )

}

export default forwardRef(TablePack)