import React, { useRef, useState, forwardRef, useImperativeHandle, useMemo, useLayoutEffect } from 'react'
import { message, Modal, Button, Popover } from 'antd'
import { ExclamationCircleOutlined } from '@ant-design/icons'
import QRcode from 'qrcode.react'
import Generator, { defaultSettings, defaultCommonSettings, defaultGlobalSettings } from 'fr-generator'
import { settings, baseSettings, globalSettings } from './config'
import widgets from '../widgets'
import { saveTableConfig, GetTableConfigJson, ReloadTableFields } from '../../apis/process'
import FormRender from '../FormRender'
import { isObject, getVerify, setFieldJson, getNanoid } from '../../utils'
import Drag from '../components/Drag'
import styles from '../../main.less'

const FormDesigner = (props, ref) => {

  useImperativeHandle(ref, () => (
    {
      clear,
      preview,
      submit,
      initLayout,
    }
  ))

  const { tableName, extra } = props
  const [visible, setVisible] = useState(false)
  const [schema, setSchema] = useState({})
  const [fieldName, setFieldName] = useState([])
  const [textShow, setTextShow] = useState(false)
  const [open, setOpen] = useState(false)
  const designerRef = useRef(null)
  const formRenderRef = useRef(null)

  const settingsParent = useMemo(() => {
    let settingsParent = []
    settings.forEach(v => {
      v.widgets.forEach(s => {
        s.schema.tableNameParent = tableName
      })
      settingsParent.push(v)
    })
    return settingsParent
  }, [settings, tableName])

  const getJSON = (json, fieldName) => {
    let nanoid = getNanoid()
    let properties = json?.properties
    let parent = {}
    let paths = []
    let title = '(未分组)'
    let ungroupedChild = {}
    if (isObject(properties)) {
      for (let v in properties) {
        if (properties[v].type === 'object') {
          parent[v] = properties[v]
          if (v !== '(未分组)') {
            paths.push(v)
          } else {
            title = properties[v].title
          }
          let child = properties[v]?.properties
          if (isObject(child)) {
            for (let s in child) {
              let IsSystemField = false
              fieldName.forEach(j => {
                if (j.name === s) {
                  IsSystemField = j.IsSystemField
                }
              })
              child[s].tableNameParent = tableName
              child[s].tableTypeParent = json.tableType
              child[s].IsSystemField = IsSystemField
            }
          }
        } else {
          ungroupedChild[v] = {
            ...properties[v],
            tableTypeParent: json.tableType,
            tableNameParent: tableName
          }
        }
      }
    }
    if (JSON.stringify(ungroupedChild) !== '{}') {
      parent[nanoid] = {
        title: title,
        type: 'object',
        collapsed: false,
        properties: {
          ...ungroupedChild
        }
      }
      paths.push(nanoid)
    }
    return {
      ...json,
      paths,
      properties: parent,
    }
  }

  const getTableConfig = async (tableName) => {
    const { code, data, msg } = await GetTableConfigJson({ tableName })
    const res = await ReloadTableFields({ tableName })
    if (code === 0 && res.data && res.data.root) {
      if (data && typeof data === 'string' && Array.isArray(res.data.root)) {
        setFieldName(res.data.root)
        let json = getJSON(JSON.parse(data), res.data.root)
        designerRef?.current?.setValue(json)
        setSchema(json)
      }
    } else {
      message.error(msg)
    }
  }

  useLayoutEffect(() => {
    if (tableName) {
      getTableConfig(tableName)
    }
    sessionStorage.setItem('FormRender', 'preview')
    return () => {
      sessionStorage.removeItem('FormRender')
    }
  }, [tableName])

  const clear = () => {
    Modal.confirm({
      title: '确定要清空该表单配置吗?',
      icon: <ExclamationCircleOutlined />,
      centered: true,
      cancelText: '取消',
      okText: '确定',
      onCancel: () => {
        console.log('取消清空')
      },
      onOk: () => {
        const schema = designerRef.current.getValue()
        const { tableName, properties: parent } = schema
        let parentObj = {}
        if (tableName?.includes('设备_台账')) {
          for (let v in parent) {
            if (isObject(parent)) {
              let child = parent[v].properties
              if (isObject(child)) {
                for (let s in child) {
                  if (s === '编码') {
                    parentObj = {
                      [v]: {
                        ...parent[v],
                        properties: {
                          [s]: child[s]
                        }
                      },
                    }
                  }
                }
              }
            }
          }
        }
        let newschema = {
          ...schema,
          properties: parentObj
        }
        designerRef.current.setValue(newschema)
      }
    })
  }

  const preview = () => {
    let json = designerRef.current.getValue()
    let verify = getVerify(json)
    if (verify === true) {
      setTextShow(false)
      setVisible(true)
      setSchema(json)
    }
  }

  const submit = async () => {
    let json = designerRef.current.getValue()
    let verify = getVerify(json)
    if (verify === true) {
      const { code, data, msg } = await saveTableConfig(json)
      if (code === 0) {
        message.info('保存成功')
      } else {
        message.error(msg)
      }
    }
  }

  const initLayout = () => {
    let json = designerRef.current.getValue()
    let verify = getVerify(json)
    if (verify === true) {
      Modal.confirm({
        title: '是否确定使用剩余附加字段一键生成表单?',
        icon: <ExclamationCircleOutlined />,
        centered: true,
        cancelText: '取消',
        okText: '确定',
        onCancel: () => {
          console.log('取消一键生成')
        },
        onOk: () => {
          designerRef.current.setValue(setFieldJson(json, fieldName))
        }
      })
    }
  }

  const extraButtons = useMemo(() => {
    if (extra) {
      return [
        false,
        false,
        true,
        true,
        {
          text: '清空',
          onClick: clear
        },
        {
          text: '一键布局',
          onClick: initLayout,
        },
        {
          text: '预览',
          onClick: preview
        },
        {
          text: '提交',
          type: 'primary',
          onClick: submit,
        },
      ]
    }
    return [false, false, false, false]
  }, [extra, fieldName])

  const getFields = (schema) => {
    let array = []
    let paths = []
    let inError = false
    let outError = false
    let parent = schema?.properties
    let parentObj = {}
    if (isObject(parent)) {
      for (let v in parent) {
        if (parent[v].type === 'object') {
          paths.push(v)
          parentObj[v] = parent[v]
          let child = parent[v]?.properties
          if (isObject(child)) {
            let childObj = {}
            for (let s in child) {
              if (child[s].type !== 'object') {
                array.push({ ...child[s], fieldName: s })
                childObj[s] = child[s]
              } else {
                inError = true
                let sun = child[s]?.properties
                if (isObject(sun)) {
                  for (let k in sun) {
                    array.push({ ...sun[k], fieldName: k })
                    childObj[k] = sun[k]
                  }
                }
              }
            }
            parentObj[v].properties = childObj
          }
        } else {
          outError = true
          let nanoid = getNanoid()
          paths.push(nanoid)
          parentObj[nanoid] = {
            title: '(未分组)',
            type: 'object',
            collapsed: false,
            properties: {
              [v]: parent[v]
            }
          }
        }
      }
    }
    return {
      inError,
      outError,
      fields: array,
      newSchema: {
        ...schema,
        paths,
        properties: parentObj
      }
    }
  }

  const onSchemaChange = (schema) => {
    let { inError, outError, fields, newSchema } = getFields(schema)
    window.designer = fields
    if (inError || outError) {
      designerRef.current.setValue(newSchema)
    }
  }

  const testSubmit = async () => {
    const { formValue, relationForm, errors } = await formRenderRef?.current?.getValues()
    if (errors.length) {
      setTextShow(false)
      return message.error('表单校验未通过!')
    } else {
      setTextShow(true)
    }
  }

  const canDelete = (values) => {
    const { $id, tableTypeParent, IsSystemField, title, type } = values
    if (tableTypeParent === '物联设备表' && IsSystemField) {
      if (fieldName.some(v => v.name === $id)) {
        message.info(`表类型为:【物联设备表】内部字段【${$id}】不允许删除!`)
        return false
      }
    }
    if (type === 'object' && ($id === '#/物联数据' || $id === '物联数据')) {
      message.info(`表类型为:【物联设备表】包含内部字段不允许删除`)
      return false
    }
    return true
  }

  const onOpenChange = (value) => {
    console.log('value', value)
  }

  if (!tableName) {
    return null
  }

  return (
    <div className={styles.pandaXform} style={{ height: '100%' }}>
      <div style={{ width: '100%', height: '100%' }}>
        <Generator
          // configProvider={{ prefixCls: prefixClsPandaXform }}
          canDelete={canDelete}
          mapping={{
            object: 'Header',
          }}
          validation={false}
          ref={designerRef}
          extraButtons={extraButtons}
          widgets={widgets}
          settings={settingsParent}
          commonSettings={baseSettings}
          globalSettings={globalSettings}
          onSchemaChange={onSchemaChange}
        />
        <Drag
          title={tableName}
          width='80%'
          bodyStyle={{ height: '700px', overflow: 'auto' }}
          onOk={() => setVisible(false)}
          onCancel={() => setVisible(false)}
          visible={visible}
          getContainer={false}
          destroyOnClose
          footer={
            <div>
              <span style={{ color: '#18b918', fontSize: '14px', paddingRight: '10px', display: textShow ? 'inline-block' : 'none' }}>该表单内容校验通过,可正常使用!</span>
              <Button onClick={() => setVisible(false)}>取消</Button>
              <Popover
                trigger="click"
                open={open}
                onOpenChange={onOpenChange}
                content={
                  <div>
                    <QRcode value={`${window.origin}/civ_mobile/product/workflow/formTest?env=dev&tableName=${tableName}`} />
                  </div>
                }
              >
                <Button>移动端预览</Button>
              </Popover>
              <Button type='primary' onClick={testSubmit}>测试提交</Button>
            </div>
          }
        >
          <FormRender
            ref={formRenderRef}
            schemaValues={{ formJson: schema }}
          // codes={{
          //   工单编号: '12421413431',
          //   事件编号: 'PO230000013',
          // }}
          />
        </Drag>
      </div>
    </div>
  )

}

export default forwardRef(FormDesigner)