import React, { useEffect, useMemo, useState } from 'react'
import { Input, Modal, Tree, message, Table } from 'antd'
import { UserAddOutlined, DownOutlined } from '@ant-design/icons'
import { getUserSelector } from '../../../../../apis/process'
import styles from '../index.less'
import Drag from '../../../../components/Drag'

const columns = [
  {
    title: '序号',
    render: (r, t, i) => i + 1
  },
  {
    title: '用户名',
    dataIndex: 'loginName',
    key: 'loginName',
  },
  {
    title: '用户名',
    dataIndex: 'userName',
    key: 'userName',
  }
]

const getUserList = (treeData, treeKey, search) => {
  let array = []
  if (Array.isArray(treeData)) {
    treeData.forEach(v => {
      if (v.ouCode === treeKey[0]) {
        if (Array.isArray(v.userList)) {
          array.push(...v.userList)
        }
      }
      array.push(...getUserList(v.children, treeKey))
    })
  }
  return search ? array.filter(v => v.userName.includes(search)) : array
}

const getSelectUser = (treeData, selectKeys, property) => {
  let array = []
  if (Array.isArray(treeData)) {
    treeData.forEach(v => {
      if (Array.isArray(v.userList)) {
        v.userList.forEach(s => {
          if (selectKeys.includes(s[property])) {
            array.push(s)
          }
        })
      }
      array.push(...getSelectUser(v.children, selectKeys, property))
    })
  }
  return array
}

const UserTree = (props) => {

  const userID = window?.globalConfig?.userInfo?.OID || 1
  const { value, onChange, schema, addons } = props
  const { disabled, presetValue, isMultiple, placeholder, role, isMySite, isStoreID } = schema
  const property = isStoreID ? 'userID' : 'userName'
  const [visible, setVisible] = useState(false)
  const [treeData, setTreeData] = useState([])
  const [treeKey, setTreeKey] = useState([])
  const [selectKeys, setSelectKeys] = useState([])
  const [search, setSearch] = useState('')
  const userList = useMemo(() => {
    return getUserList(treeData, treeKey, search)
  }, [treeData, treeKey, search])

  const selectUser = useMemo(() => {
    return getSelectUser(treeData, selectKeys, property)
  }, [treeData, selectKeys, property])

  const valueShow = useMemo(() => {
    if (isStoreID && value) {
      return getSelectUser(treeData, value, 'userID').map(v => v.userName)
    }
    return value
  }, [treeData, value, isStoreID])

  const inputFocus = async () => {
    if (addons) {
      setVisible(true)
      getData()
    }
  }

  const getData = async () => {
    const { code, data, msg } = await getUserSelector(role, isMySite ? userID : false)
    if (code === 0) {
      setTreeData(data.treeList)
      setSelectKeys(value ? value.split(',') : [])
    } else {
      message.error(msg)
    }
  }

  useEffect(() => {
    if (addons) {
      addons.setValue(addons.dataPath, presetValue)
    } else {
      onChange(presetValue)
    }
  }, [presetValue])

  useEffect(() => {
    if (isStoreID) {
      getData()
    }
  }, [])

  const onOk = () => {
    onChange(selectUser.map(v => v[property]).join(','))
    setVisible(false)
  }

  const onCancel = () => {
    setVisible(false)
  }

  const treeChange = (value) => {
    setTreeKey(value)
  }

  return (
    <div className={styles.userTree}>
      <Input
        disabled={disabled}
        value={valueShow}
        placeholder={disabled ? (placeholder || '') : (placeholder || '点击选择人员')}
        onClick={inputFocus}
        addonAfter={disabled ? null : <UserAddOutlined style={{ color: 'rgba(0, 0, 0, 0.25)' }} onClick={inputFocus} />}
        style={{ width: '100%' }}
      />
      <Drag
        width='60%'
        title='人员选择器'
        visible={visible}
        bodyStyle={{ height: '600px' }}
        onCancel={onCancel}
        onOk={onOk}
      >
        <div className={styles.box}>
          <div className={styles.left}>
            <div className={styles.title}>部门列表</div>
            <div className={styles.tree}>
              <Tree
                value={treeKey}
                fieldNames={{ title: 'ouName', key: 'ouCode' }}
                blockNode
                showIcon
                defaultExpandAll
                defaultSelectedKeys={['0-0-0']}
                switcherIcon={<DownOutlined />}
                treeData={treeData}
                onSelect={treeChange}
              />
            </div>
          </div>
          <div className={styles.right}>
            <div className={styles.title}>
              <Input.Search
                value={search}
                allowClear
                onChange={e => setSearch(e.target.value)}
                placeholder='请输入姓名进行查找'
                style={{ width: '300px' }}
              />
            </div>
            <div className={styles.list}>
              <div className={styles.userList}>
                <div className={styles.userStatus}>待选人员</div>
                <Table
                  rowKey={r => r[property]}
                  size='small'
                  columns={columns}
                  dataSource={userList}
                  pagination={false}
                  bordered
                  rowSelection={{
                    type: isMultiple ? 'checkbox' : 'radio',
                    selectedRowKeys: selectKeys,
                    onChange: (select) => {
                      if (isMultiple) {
                        let noSelectUser = userList.filter(v => !select.includes(v[property])).map(v => v[property])
                        let users = selectKeys.concat(select)
                        let keys = users.filter(v => !noSelectUser.includes(v))
                        setSelectKeys(keys)
                      } else {
                        setSelectKeys(select)
                      }
                    }
                  }}
                  scroll={{ y: 180 }}
                />
              </div>
              <div className={styles.selectList}>
                <div className={styles.userStatus}>已选人员</div>
                <Table
                  rowKey={r => r[property]}
                  rowSelection={{
                    type: isMultiple ? 'checkbox' : 'radio',
                    selectedRowKeys: selectKeys,
                    onChange: (select) => {
                      setSelectKeys(select)
                    }
                  }}
                  size='small'
                  columns={columns}
                  dataSource={selectUser}
                  pagination={false}
                  bordered
                  scroll={{ y: 180 }}
                />
              </div>
            </div>
          </div>
        </div>
      </Drag>
    </div>
  )

}

export default UserTree